@@ -8,6 +8,8 @@
 #undef HAVE_PTHREADS
 #undef ksize_t
 #undef HAVE_GTKHTML
-#define USE_DAV
 #undef USE_GTKEXTEXT
 #undef HAVE_GUILE
+#undef USE_GCONF
+#undef HAVE_GNOME_VFS
+#define I_LIKE_WASTING_RESOURCES 0
