@@ -3,12 +3,16 @@
 
 gboolean copy_file( const gchar *source, const gchar *dest );
 gboolean copy_dir( const char *source, const gchar *dest, gboolean move );
-gboolean move_file( gchar *source, gchar *dest );
+gboolean move_file( const gchar *source, const gchar *dest );
 gboolean delete_file( const gchar *file );
 gboolean delete_dir( const gchar *path );
 gboolean mkdir_recursive( const gchar *path );
 
-gchar *relative_path( gchar *text, const gchar *root );
-gchar *relative_to_full( gchar *relPath );
+gchar *relative_path( const gchar *text, const gchar *root );
+gchar *relative_to_full( const gchar *relPath );
+
+gboolean overwrite( const gchar *filename );
+
+gchar* paths_match( const gchar *path1, const gchar *path2 );
 
 #endif
