@@ -1,9 +1,10 @@
 #ifndef HELPERS_H
 #define HELPERS_H
 
-typedef struct _Helper {
+typedef struct Helper {
 	gchar *path;
 	gchar *mime_type;
+	gchar *icon;
 } Helper;
 
 
@@ -19,4 +20,9 @@ void screem_helper_execute( gchar *path );
 
 GtkWidget *screem_helper_menu( void );
 
+void screem_helpers_build_toolbar( void );
+
+void edit_helper_scripts( void );
+
+
 #endif
