@@ -40,6 +40,18 @@ void screem_browse_list_show_dir( gchar *dir );
 void browse_list_clicked( GtkWidget *widget, GdkEventButton *event,
 			  gpointer data );
 
-void resource_list_display( void );
+void resource_list_display( GtkWidget *tree );
+
+void tree_file_info( void );
+
+void create_dir_callback( gchar *string, gpointer data );
+void screem_site_create_dir( void );
+
+void exclude_file( GtkWidget *widget );
+void ignore_file( GtkWidget *widget );
+void ascii_file( GtkWidget *widget );
+
+void file_information( const gchar *filename );
+
 
 #endif
