@@ -1,16 +1,16 @@
 #ifndef SCREEM_TODO_H
 #define SCREEM_TODO_H
 
-typedef struct _Todo_item {
+typedef struct TodoItem {
         gchar *task;
         gchar *assigned;
         gchar *priority;
         gchar *linked_to;
         gchar *description;
-} Todo_item;
+} TodoItem;
 
-Todo_item* screem_todo_item_new( void );
-void screem_todo_item_destroy( Todo_item *item );
+TodoItem* screem_todo_item_new( void );
+void screem_todo_item_destroy( TodoItem *item );
 void todo_view( void );
 
 #endif
