@@ -35,7 +35,6 @@
 
 extern GtkWidget *app;
 extern Site *current_site;
-extern Page *current_page;
 
 /* Widgets that are accessed from ssiWizard_handlers.c */
 GtkWidget *window1;
@@ -84,8 +83,8 @@ init()
 {
    GnomeUIInfo menuinfo[] = { 
 	{
-	   GNOME_APP_UI_ITEM, N_( "SSI directive..." ),
-	   N_( "Insert a Server-side Include" ),
+	   GNOME_APP_UI_ITEM, N_("SSI directive..."),
+	   N_("Insert a Server-side Include"),
 	   ssiWizard_create, NULL, NULL,
 	   GNOME_APP_PIXMAP_STOCK,
 	   GNOME_STOCK_MENU_BLANK,
@@ -187,11 +186,8 @@ ssiWizard_create(void)
 
   Page *page;
 
-  if( current_site )
-	  page = screem_site_get_current_page( current_site );
-  else
-	  page = current_page;
-
+  page = screem_site_get_current_page( current_site );
+  
   g_return_if_fail( page != NULL );
    
    if (window1) {
