@@ -23,27 +23,13 @@
  *  about box under the help menu for a contact address
  */
 
-void upload_button_clicked                  (GtkButton       *button,
-					     gpointer         user_data);
-
-void error_button_clicked                  (GtkButton       *button,
-					    gpointer         user_data);
-
-void close_button_clicked                   (GtkButton       *button,
-					     gpointer         user_data);
+enum site_op
+{
+    site_op_update, site_op_fetch, site_op_resync
+};
 
 void create_update_dialog (void);
 void create_error_log_window (void);
 void uploadSite(void);
 void *update_thread(void *bler);
 
-void fe_connection( fe_conn_status status );
-bool fe_can_update (const struct site_file_t *file);
-void fe_updating (const struct site_file_t *file);
-void fe_updated (const struct site_file_t *file, const bool success,
-		 const char *error);
-void fe_transfer_progress (size_t progress, size_t total);
-void fe_synching( const struct site_file_t *file );
-void fe_synched( const struct site_file_t *file, const bool success,
-		const char *error );
-void fe_fetch_found( const struct site_file_t *file );
