@@ -1,7 +1,7 @@
 /*  Screem:  cvs.c,
  *  CVS features for Screem
  *
- *  Copyright (C) 1999  David A Knight
+ *  Copyright (C) 1999, 2000  David A Knight
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,16 +28,15 @@
 #include "editor.h"
 #include "fileops.h"
 #include "page.h"
+#include "pageUI.h"
 #include "site.h"
 #include "siteTreeUI.h"
 #include "siteUI.h"
 #include "support.h"
 
 extern Site *current_site;
-extern Page *current_page;
 
 static gboolean cvs_flags_dialog( gchar **extra_flags, gchar **log_text );
-static gboolean execute_cvs_command( gchar *command );
 
 static GtkWidget *settings_dialog( gboolean message );
 static GtkWidget *checkout_dialog( void );
@@ -48,7 +47,6 @@ gboolean cvs_delete_with_filename( gchar *path )
    	const gchar *cvs_root;
 	gchar *command;
 
-	gchar *log_text;
 	gchar *extra_flags;
 	gboolean ret;
 
@@ -70,7 +68,7 @@ gboolean cvs_delete_with_filename( gchar *path )
 	ret = execute_command( command );
 	g_free( command );
 
-	return ret;
+	return (ret == 0);
 }
 
 gboolean cvs_add_with_filename( gchar *path )
@@ -101,7 +99,7 @@ gboolean cvs_add_with_filename( gchar *path )
 	ret = execute_command( command );
 	g_free( command );
 
-	return ret;
+	return (ret==0);
 }
 
 gboolean cvs_update_page()
@@ -117,11 +115,8 @@ gboolean cvs_update_page()
 
 	site = current_site;
 
-	if( site )
-		page = screem_site_get_current_page( site );
-	else
-		page = current_page;
-
+	page = screem_site_get_current_page( site );
+	
 	g_return_val_if_fail( page != NULL, FALSE );
 
 	cvs_root = screem_site_get_cvs_root( site );
@@ -141,15 +136,12 @@ gboolean cvs_update_page()
 	g_free( command );
 
 	/* reload the page now */
-	if( ret ) {
-		g_free( page->data );
-		page->data = NULL;
-		screem_page_load( page );
-		screem_editor_clear();
-		screem_editor_insert( 0, page->data );
+	if( ret == 0 ) {
+		screem_page_revert( page );  /* does a reload */
+		screem_page_insert( page );
 	}
 
-	return ret;
+	return (ret==0) ;
 }
 
 gboolean cvs_commit_page()
@@ -166,11 +158,8 @@ gboolean cvs_commit_page()
 
 	site = current_site;
 
-	if( site )
-		page = screem_site_get_current_page( site );
-	else
-		page = current_page;
-
+	page = screem_site_get_current_page( site );
+	
 	g_return_val_if_fail( page != NULL, FALSE );
 
 	cvs_root = screem_site_get_cvs_root( site );
@@ -193,7 +182,7 @@ gboolean cvs_commit_page()
 
 	g_free( command );
 
-	return ret;
+	return (ret==0);
 }
 
 gboolean cvs_checkout_site()
@@ -201,10 +190,11 @@ gboolean cvs_checkout_site()
 	GtkWidget *dialog;
 	GtkWidget *module_name;
 	GtkWidget *cvs_flags;
+	GtkWidget *cvs_repository;
 	gint button;
 	gchar *module;
 	gchar *extra_flags;
-	const gchar *cvs_root;
+	gchar *cvs_root;
 	gchar *command;
 	gboolean ret;
 
@@ -212,27 +202,39 @@ gboolean cvs_checkout_site()
 
 	button = gnome_dialog_run( GNOME_DIALOG( dialog ) );
 
-	module_name = gtk_object_get_data( GTK_OBJECT( dialog ),"module_name");
-	cvs_flags = gtk_object_get_data( GTK_OBJECT( dialog ), "cvs_flags" );
+	module_name = (GtkWidget*)
+		gtk_object_get_data( GTK_OBJECT( dialog ),"module_name");
+	cvs_flags = (GtkWidget*)
+		gtk_object_get_data( GTK_OBJECT( dialog ), "cvs_flags" );
+	cvs_repository = (GtkWidget*)
+		gtk_object_get_data( GTK_OBJECT( dialog ), "cvs_root" );
 
 	module = gtk_editable_get_chars( GTK_EDITABLE( module_name ), 0, -1 );
 	extra_flags = g_strdup( gtk_entry_get_text( GTK_ENTRY( cvs_flags ) ) );
+	cvs_root = g_strdup( gtk_entry_get_text( GTK_ENTRY( cvs_repository ) ) );
 
 	gtk_widget_destroy( dialog );
 
 	if( button == 1 ) {
 		g_free( module );
 		g_free( extra_flags );
+		g_free( cvs_root );
 		return FALSE;
 	}
 
 	/* ok'ed */
-	command = g_strdup_printf( "cvs -z3 checkout %s \"%s\" 2>&1",
-				   extra_flags, module );
+	if(strlen(cvs_root) != 0)
+		command = g_strdup_printf( "cvs -d %s -z3 checkout %s \"%s\" 2>&1",
+					   cvs_root, extra_flags, module );
+	else
+		command = g_strdup_printf( "cvs -z3 checkout %s \"%s\" 2>&1",
+					   extra_flags, module );
      	g_free( extra_flags );
+     	g_free( cvs_root );
 	ret = execute_command( command );
+	g_free( command );
 
-	if( ! ret ) {
+	if( ret != 0 ) {
 		g_free( module );
 		return FALSE;
 	}
@@ -241,7 +243,7 @@ gboolean cvs_checkout_site()
 	ret = screem_site_open_with_filename( module );
 	g_free( module );
 
-	return ret;
+	return (ret==0);
 }
 
 gboolean cvs_update_site()
@@ -274,7 +276,7 @@ gboolean cvs_update_site()
 
 	g_free( command );
 
-	return ret;
+	return (ret==0);
 }
 
 gboolean cvs_import_site()
@@ -293,7 +295,7 @@ gboolean cvs_import_site()
 	gboolean cpok;
 
 	GList *list;
-	gchar *new;
+	gchar *new_path;
 	const gchar *pathname;
 	Page *page;
 	gchar cwd[16384];
@@ -321,8 +323,11 @@ cvs -d %s import %s -m \"%s\" \"%s\" screem start 2>&1",
 
 	g_free( command );
 
-	if( ! ret )
+	if( ret != 0 ) {
+		chdir( screem_site_get_pathname( site ) );
 		return FALSE;
+	}
+
 
 	/* backup directory and perform a checkout on the imported site */
 	backup_path = tmpnam( NULL );
@@ -336,12 +341,13 @@ cvs -d %s import %s -m \"%s\" \"%s\" screem start 2>&1",
 		/* perform cvs checkout */
 		command = g_strdup_printf("cvs -d %s -z3 checkout \"%s\" 2>&1",
 					  cvs_root, name );
-		cpok = execute_command( command );
+		cpok = ( execute_command( command ) == 0 );
+		g_free( command );
 		if( ! cpok ) {
 			/* failed checkout: restore from backup */
 			copy_dir( backup_path, path, FALSE );
 		}
-
+		
 		/* delete backup */
 		delete_dir( backup_path );
 	}
@@ -357,10 +363,11 @@ cvs -d %s import %s -m \"%s\" \"%s\" screem start 2>&1",
 		     list = list->next ) {
 			page = (Page*)list->data;
 			pathname = screem_page_get_pathname( page );
-			new = g_strdup_printf( "%s%c%s", cwd,  G_DIR_SEPARATOR,
-					       pathname + strlen( path ) );
-			screem_page_set_pathname( page, new );
-			g_free( new );
+			new_path = g_strdup_printf( "%s%c%s", cwd, 
+						    G_DIR_SEPARATOR,
+						    pathname + strlen( path ));
+			screem_page_set_pathname( page, new_path );
+			g_free( new_path );
 		}
 		refresh_file_tree();
 		screem_site_save( site );
@@ -368,7 +375,7 @@ cvs -d %s import %s -m \"%s\" \"%s\" screem start 2>&1",
 
 	chdir( screem_site_get_pathname( site ) );
 
-	return ret;
+	return cpok;
 }
 
 gboolean cvs_commit_site()
@@ -402,7 +409,7 @@ gboolean cvs_commit_site()
 
 	g_free( command );
 
-	return ret;
+	return (ret==0);
 }
 
 static gboolean cvs_flags_dialog( gchar **extra_flags, gchar **log_text )
@@ -418,8 +425,10 @@ static gboolean cvs_flags_dialog( gchar **extra_flags, gchar **log_text )
 
 	button = gnome_dialog_run( GNOME_DIALOG( dialog ) );
 
-	message = gtk_object_get_data( GTK_OBJECT( dialog ), "message" );
-	cvs_flags = gtk_object_get_data( GTK_OBJECT( dialog ), "cvs_flags" );
+	message = (GtkWidget*)
+		gtk_object_get_data( GTK_OBJECT( dialog ), "message" );
+	cvs_flags = (GtkWidget*)
+		gtk_object_get_data( GTK_OBJECT( dialog ), "cvs_flags" );
 
 	if( log_text )
 		*log_text = gtk_editable_get_chars( GTK_EDITABLE( message ),
@@ -543,6 +552,9 @@ static GtkWidget *checkout_dialog()
   GtkWidget *frame2;
   GtkWidget *entry1;
   GtkWidget *cvs_flags;
+  GtkWidget *frame3;
+  GtkWidget *entry3;
+  GtkWidget *cvs_root;
   GtkWidget *dialog_action_area1;
   GtkWidget *button1;
   GtkWidget *button2;
@@ -602,6 +614,26 @@ static GtkWidget *checkout_dialog()
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (cvs_flags);
 
+  frame3 = gtk_frame_new (_("CVS Root"));
+  gtk_widget_ref (frame3);
+  gtk_object_set_data_full (GTK_OBJECT (dialog), "frame3", frame3,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (frame3);
+  gtk_box_pack_start (GTK_BOX (vbox1), frame3, FALSE, FALSE, 0);
+
+  entry3 = gnome_entry_new ("cvs_root_hist");
+  gtk_widget_ref (entry3);
+  gtk_object_set_data_full (GTK_OBJECT (dialog), "entry3", entry3,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (entry3);
+  gtk_container_add (GTK_CONTAINER (frame3), entry3);
+
+  cvs_root = gnome_entry_gtk_entry (GNOME_ENTRY (entry3));
+  gtk_widget_ref (cvs_root);
+  gtk_object_set_data_full (GTK_OBJECT (dialog), "cvs_root", cvs_root,
+                            (GtkDestroyNotify) gtk_widget_unref);
+  gtk_widget_show (cvs_root);
+
   dialog_action_area1 = GNOME_DIALOG (dialog)->action_area;
   gtk_object_set_data (GTK_OBJECT (dialog), "dialog_action_area1", dialog_action_area1);
   gtk_widget_show (dialog_action_area1);
