@@ -2,7 +2,7 @@
  *  The UI side of the site tree, handling user interaction such as clicking,
  *  and drag and drop
  *
- *  Copyright (C) 1999  David A Knight
+ *  Copyright (C) 1999, 2000  David A Knight
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,8 +29,18 @@
 #include <gnome-xml/debugXML.h>
 #include <gnome-xml/tree.h>
 #include <gnome-xml/parser.h>
+#include <glade/glade.h>
 
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
+#include <pwd.h>
+#include <grp.h>
+
+#ifdef HAVE_GNOME_VFS
+#include <libgnomevfs/gnome-vfs-mime.h>
+#endif
 
 #include "cvs.h"
 #include "editor.h"
@@ -38,11 +48,13 @@
 #include "page.h"
 #include "pageUI.h"
 #include "preview.h"
+#include "preferences.h"
 #include "site.h"
 #include "siteTree.h"
 #include "siteTreeUI.h"
 #include "siteUI.h"
 #include "support.h"
+#include "tag_tree.h"
 #include "xml.h"
 
 #include "dir-open.xpm"
@@ -50,6 +62,7 @@
 
 extern Site *current_site;
 extern GtkWidget *app;
+extern Preferences *cfg;
 
 extern GList *icon_list;
 
@@ -59,45 +72,15 @@ static void add_node( Site *site, GtkCTree *t, xmlDocPtr doc, xmlNodePtr node,
 
 static void tree_item_clicked(GtkCTree *tree, GtkCTreeNode *row, gint column);
 
-static void tree_open_file( void );
-static void tree_delete_file( void );
-static void tree_rename_file( void );
+void tree_open_file( void );
+void tree_delete_file( void );
+void tree_rename_file( void );
 
 static GdkPixmap *folder_open = NULL;
 static GdkPixmap *folder_close = NULL;
 static GdkBitmap *folder_open_mask = NULL;
 static GdkBitmap *folder_close_mask = NULL;
 
-static GnomeUIInfo file_tree_menu[] = {
-        GNOMEUIINFO_MENU_NEW_ITEM( N_("_New Page..."), 
-                                   N_("Add a new page to the site"), 
-                                   GTK_SIGNAL_FUNC( screem_page_create_page ),
-				   NULL ),
-	GNOMEUIINFO_MENU_NEW_ITEM( N_( "_New Blank Page" ),
-				   N_("Create a blank page (add if working on a site)" ),
-				   GTK_SIGNAL_FUNC( screem_page_create_blank_page ),
-				   NULL ),
-        { GNOME_APP_UI_ITEM, N_("_Delete File"), 
-          N_( "Delete File" ),
-          GTK_SIGNAL_FUNC( tree_delete_file ), NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0,
-          GDK_CONTROL_MASK, NULL },
-        { GNOME_APP_UI_ITEM, N_("_Rename File"), 
-          N_( "Rename File" ),
-          GTK_SIGNAL_FUNC( tree_rename_file ), NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK, 0,
-          GDK_CONTROL_MASK, NULL },
-        { GNOME_APP_UI_ITEM, N_("_Open"), N_( "Opens the file" ),
-          GTK_SIGNAL_FUNC( tree_open_file ), NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 'O',
-          GDK_CONTROL_MASK, NULL },
-        { GNOME_APP_UI_ITEM, N_("Refresh"), N_( "Refresh the tree" ),
-          GTK_SIGNAL_FUNC( refresh_file_tree ), NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH, 0,
-          GDK_CONTROL_MASK, NULL },
-        GNOMEUIINFO_END
-};
-
 static GnomeUIInfo tree_dnd_menu[] = {
         { GNOME_APP_UI_ITEM, N_("_Move here"), N_("Move file"),
           0, NULL, NULL,
@@ -117,19 +100,29 @@ static GnomeUIInfo tree_dnd_menu[] = {
 
 static void load_folder_icons()
 {
-	GdkImlibImage *image;
+	GdkPixbuf *pixbuf;
 
 	if( ! folder_open ) {
-		/* we haven't created the dir icon the file tree yet */
-		image = gdk_imlib_create_image_from_xpm_data( DIRECTORY_OPEN_XPM );
-		gdk_imlib_render( image, image->rgb_width, image->rgb_height );
-		folder_open = gdk_imlib_move_image( image );
-		folder_open_mask = gdk_imlib_move_mask( image );
-		
-		image = gdk_imlib_create_image_from_xpm_data( DIRECTORY_CLOSE_XPM );
-		gdk_imlib_render( image, image->rgb_width, image->rgb_height );
-		folder_close = gdk_imlib_move_image( image );
-		folder_close_mask = gdk_imlib_move_mask( image );
+		pixbuf = gdk_pixbuf_new_from_xpm_data( (const char**)
+						       DIRECTORY_OPEN_XPM );
+		if( ! pixbuf ) {
+			g_warning( _( "Failed to create folder open icon" ) );
+			return;
+		}
+		gdk_pixbuf_render_pixmap_and_mask( pixbuf, &folder_open,
+						   &folder_open_mask,
+						   100 );
+		gdk_pixbuf_unref( pixbuf );
+		pixbuf = gdk_pixbuf_new_from_xpm_data( (const char**)
+						       DIRECTORY_CLOSE_XPM );
+		if( ! pixbuf ) {
+			g_warning( _( "Failed to create folder close icon" ) );
+			return;
+		}
+		gdk_pixbuf_render_pixmap_and_mask( pixbuf, &folder_close,
+						   &folder_close_mask,
+						   100 );
+		gdk_pixbuf_unref( pixbuf );
 	}
 }
 
@@ -137,23 +130,23 @@ static void build_file_tree( GtkWidget *tree, Site *site, xmlDocPtr doc )
 {
 	xmlNodePtr node;
  	GtkCTreeNode *tnode;
-
+	
 	g_return_if_fail( tree != NULL );
-        g_return_if_fail( doc != NULL );
-
+	g_return_if_fail( doc != NULL );
+	
 	load_folder_icons();
 
 	node = doc->root;
 	/* not interested in the xml version element */
-        node = node->childs;
+	node = node->childs;
 
 	gtk_clist_freeze( GTK_CLIST( tree ) );
 	add_node( site, GTK_CTREE( tree ), doc, node, NULL );
-
+	
 	/* now sort the tree */
 	tnode = gtk_ctree_node_nth( GTK_CTREE( tree ), 0 );
-        gtk_ctree_sort_recursive( GTK_CTREE( tree ), tnode );
-        gtk_ctree_expand( GTK_CTREE( tree ), tnode );
+	gtk_ctree_sort_recursive( GTK_CTREE( tree ), tnode );
+	gtk_ctree_expand( GTK_CTREE( tree ), tnode );
 
 	gtk_clist_thaw( GTK_CLIST( tree ) );
 }
@@ -168,121 +161,185 @@ static void build_file_tree( GtkWidget *tree, Site *site, xmlDocPtr doc )
  */
 void refresh_file_tree()
 {
-	Site *site;
+	static Site *site = NULL;
 	xmlDocPtr doc;
 	GtkWidget *tree;
 
 	const gchar *site_path;
 
-	site = current_site;
+     	tree = gtk_object_get_data( GTK_OBJECT( app ), "file_tree" );
 
-	g_return_if_fail( site != NULL );
+	/* fake sites can't refresh file trees */
+	if( screem_site_get_fake_flag( current_site ) ) {
+	       	gtk_clist_clear( GTK_CLIST( tree ) );
+		return;
+	}
 
-	site_path = screem_site_get_pathname( site );
+	site_path = screem_site_get_pathname( current_site );
 
-	doc = build_directory_tree( site, site_path, NULL );
+	doc = build_directory_tree( current_site, site_path, NULL );
 
 	g_return_if_fail( doc != NULL );
 
-	identify_pages( site, doc, doc->root->childs );
+	identify_pages( current_site, doc, doc->root->childs );
 
-	tree = gtk_object_get_data( GTK_OBJECT( app ), "file_tree" );
-	gtk_clist_clear( GTK_CLIST( tree ) );
-	build_file_tree( tree, site, doc );
+	if( site != current_site )
+		gtk_clist_clear( GTK_CLIST( tree ) );
+	build_file_tree( tree, current_site, doc );
+
+	xmlFreeDoc( doc );
+
+	site = current_site;
 }
-static void add_node( Site *site, GtkCTree *t, xmlDocPtr doc, xmlNodePtr node,
-		      GtkCTreeNode *parent )
+
+static GtkCTreeNode* add_node_do( Site *site, 
+				  GtkCTree* tree, GtkCTreeNode *parent,
+				  const gchar *path, gchar *mime_type )
 {
-        gchar *text[ 2 ] = { NULL, NULL };
-        GtkCTreeNode *this = parent;
-	gchar *entry;
-	gchar *temp;
-
-	const gchar *mime_type;
-	const gchar *site_path;
-
+	struct stat s;
+	gboolean is_dir;
+	gchar *temp_path;
+	gchar *root;
+	gchar flags[ 3 ] = { '.', '.', '.' };
+	gchar *text[ 3 ] = { NULL, NULL, NULL };
+	
 	GdkPixmap *open = folder_open;
         GdkBitmap *open_mask = folder_open_mask;
         GdkPixmap *close = folder_close;
         GdkBitmap *close_mask = folder_close_mask;
-
-	gboolean is_dir;
-
-	GList *list;
 	Icons *icon = NULL;
-	const gchar *file;
 
-    	g_return_if_fail( t != NULL );
+	GtkCTreeNode *node;
+	GtkCTreeNode *exist;
+	GtkCTreeNode *die;
+	GtkCTreeNode *already;
+	gchar *exist_path;
+	
+	if( ! path )
+		return parent;
+	
+	if( stat( path, &s ) < 0 )
+		return parent;
+	
+	is_dir = S_ISDIR( s.st_mode );
 
+	/* get root path, and set flags if necessary */
 	if( site )
-		site_path = screem_site_get_pathname( site );
+		root = g_strdup( screem_site_get_pathname( site ) );
 	else
-		site_path = RESOURCE_PATH;
+		root = g_strdup( RESOURCE_PATH );
+
+	/* bypass the root path for the file, which is either the site
+	   pathname, or the resource folder pathname */
+
+	if( is_dir ) {
+		temp_path = path + strlen( root );
+		temp_path[ strlen( temp_path ) - 1 ] = '\0';
+		text[ 0 ] = g_strconcat( G_DIR_SEPARATOR_S,
+					 g_basename( temp_path ), NULL );
+		temp_path -= strlen( root );
+	} else {
+		text[ 0 ] = g_strdup( g_basename( path ) );
+		icon = icon_from_mime_type( path, mime_type );
+		open = icon->open;
+		open_mask = icon->open_mask;
+		close = open;
+		close_mask = open_mask;
+	}
+
+	if( site ) {
+		if( screem_site_is_excluded( site, path ) )
+			flags[ 0 ] = 'E';
+		if( screem_site_is_ignored( site, path ) )
+			flags[ 1 ] = 'I';
+		if( screem_site_is_ascii( site, path ) )
+			flags[ 2 ] = 'A';
+		text[ 1 ] = g_strdup_printf( "%c%c%c", 
+					     flags[ 0 ], flags[ 1 ],
+					     flags[ 2 ] );
+	}
+
+	/* do we already have this node? */
+     	exist = parent;
+	if( exist )
+		exist = GTK_CTREE_ROW( exist )->children;
+	else
+		exist = gtk_ctree_node_nth( tree, 0 );
+   	already = NULL;
+	while( exist ) {
+		exist_path = gtk_ctree_node_get_row_data( tree, exist );
+		/* stat it to check if it exists, if not we remove the node */
+		if( exist_path && stat( exist_path, &s ) < 0 ) {
+			die = exist;
+			exist = GTK_CTREE_ROW( exist )->sibling;
+			gtk_ctree_remove_node( tree, die );
+		} else {
+			/* it exists, is it the one we are trying to add? */
+			if( exist_path && ! strcmp( exist_path, path ) )
+				already = exist;
+			exist = GTK_CTREE_ROW( exist )->sibling;
+		}
+	}
+
+	/* add it if we need to */
+	if( already )
+		node = already;
+	else if( strcmp( mime_type, "application/x-screem" ) ) {
+		node = gtk_ctree_insert_node( tree, parent, NULL, text, 3,
+					      close, close_mask,
+					      open, open_mask,
+					      (gboolean)parent, FALSE );
+		gtk_ctree_node_set_row_data( tree, node, 
+					     g_strdup( path ) );
+	} else
+		node = parent;
+	
+	g_free( root );
+	g_free( text[ 0 ] );
+	g_free( text[ 1 ] );
+
+	return node;
+}
+
+static void add_node( Site *site, GtkCTree *tree, 
+		      xmlDocPtr doc, xmlNodePtr node,
+		      GtkCTreeNode *parent )
+{
+	GtkCTreeNode *this;
+	gchar *mime_type;
+	gchar *entry;
+
+    	g_return_if_fail( tree != NULL );
 
         /* add the node */
         entry = xmlNodeListGetString( doc, node->childs, 1 );
 
 	mime_type = xml_get_value( node, "type" );
 
-	if( entry ) {
-		temp = entry + strlen( site_path );
-		is_dir = g_file_test( entry, G_FILE_TEST_ISDIR );
-		if( ! is_dir )
-			entry = g_strdup( g_filename_pointer( temp ) );
-		else {
-			if( strlen( temp ) > 0 ) {
-				entry = temp + strlen( temp ) - 1;
-				while( *(--entry) != G_DIR_SEPARATOR ){}
-				entry = g_strndup( entry, strlen( entry ) -1 );
-			} else {
-				entry = g_strdup( G_DIR_SEPARATOR_S );
-			}
-		}
-		text[ 0 ] = entry;
-		temp -= strlen( site_path );
+	this = add_node_do( site, tree, parent, entry, mime_type );
+	g_free( entry );
+	g_free( mime_type );
 
-		if( ! is_dir ) {
-			icon = icon_from_mime_type( mime_type );
-			
-			open = icon->open;
-			open_mask = icon->open_mask;
-			close = open;
-			close_mask = open_mask;
-		}
-
-		if( strcmp( mime_type, "application/x-screem" ) ) {
-			this = gtk_ctree_insert_node( t, parent, NULL, text,
-						      3, close, close_mask, 
-						      open, open_mask, 
-						      (gboolean)parent, FALSE);
-			/* set up its row data */
-			gtk_ctree_node_set_row_data( GTK_CTREE( t ), this,
-						     temp );
-		} else {
-			g_free( temp );
-		}
-		g_free( entry );
-	}
-
-        /* has children? */
+	/* has children? */
         if( node->childs )
-                add_node( site, t, doc, node->childs, this );
+                add_node( site, tree, doc, node->childs, this );
 
         /* move on to the next sibling */
         if( node->next )
-                add_node( site, t, doc, node->next, parent );
+                add_node( site, tree, doc, node->next, parent );
 }
 
 void tree_clicked( GtkWidget *widget, GdkEventButton *event,
 		   gpointer data )
 {
 	GtkCTreeNode *node = NULL;
+	GladeXML *xml;
         GtkWidget *menu;
+	gchar *path;
+	GtkWidget *check;
         gint row = -1;
         gint col;
 
-	static guint32 time = 0;
-
 	if( ! current_site )
 		return;
 
@@ -303,19 +360,157 @@ void tree_clicked( GtkWidget *widget, GdkEventButton *event,
 
 	/* what button was clicked? */
 	if( event->button == 1 ) {
-                if( time == event->time ) /* double click */
+                if( event->type == GDK_2BUTTON_PRESS ) /* double click */
                         tree_item_clicked( GTK_CTREE( widget ), node, 1 );
-                time = event->time;
-        } else if( event->button == 3 ) {
+	} else if( event->button == 3 ) {
                 /* select the line underneath the mouse */
-                if( node )
+                xml =  glade_xml_new( cfg->glade_path, "file_tree_menu" );
+		menu = glade_xml_get_widget( xml, "file_tree_menu" );
+
+		if( node ) {
                         gtk_ctree_select( GTK_CTREE( widget ), node );
-                menu = gnome_popup_menu_new( file_tree_menu );
+			path = gtk_ctree_node_get_row_data( GTK_CTREE( widget ), node );
+			gtk_object_set_data( GTK_OBJECT( menu ), "path",path );
+
+			/* find upload flags for the file/directory */
+			screem_set_upload_flags( current_site, xml, path );
+		} else {
+			check = glade_xml_get_widget( xml, "delete" );
+			change_state( check, NULL );
+			check = glade_xml_get_widget( xml, "rename" );
+			change_state( check, NULL );
+			check = glade_xml_get_widget( xml, "properties" );
+			change_state( check, NULL );
+			check = glade_xml_get_widget( xml, "upload_flags" );
+			change_state( check, NULL );
+		}
+		glade_xml_signal_autoconnect( xml );
+		gtk_object_set_data( GTK_OBJECT( menu ), "tree", widget );
+		gtk_object_set_data( GTK_OBJECT( menu ), "node", node );
+		gtk_object_set_data( GTK_OBJECT( widget ), "node", node );
                 gnome_popup_menu_do_popup_modal( menu, 0, 0, event, 0 );
                 gtk_widget_destroy( menu );
         }
 }
 
+void exclude_file( GtkWidget *widget )
+{
+	GladeXML *xml;
+	GtkWidget *menu;
+	gboolean active;
+	gchar *path;
+	
+	GtkWidget *tree;
+	GtkWidget *node;
+	gchar *text;
+	gchar flags[ 3 ] = { '.', '.', '.' };
+
+	xml = glade_get_widget_tree( widget );
+	menu = glade_xml_get_widget( xml, "file_tree_menu" );
+
+	tree = gtk_object_get_data( GTK_OBJECT( menu ), "tree" );
+	node = gtk_object_get_data( GTK_OBJECT( menu ), "node" );
+	path = gtk_object_get_data( GTK_OBJECT( menu ), "path" );
+
+	active = screem_site_is_excluded( current_site, path );
+
+	if( ! active )
+		screem_site_add_exclude( current_site, path );
+	else
+		screem_site_remove_exclude( current_site, path );
+
+	if( screem_site_is_excluded( current_site, path ) )
+		flags[ 0 ] = 'E';
+	if( screem_site_is_ignored( current_site, path ) )
+		flags[ 1 ] = 'I';
+	if( screem_site_is_ascii( current_site, path ) )
+		flags[ 2 ] = 'A';
+	
+	text = g_strdup_printf( "%c%c%c", flags[ 0 ], flags[ 1 ], flags[ 2 ] );
+	gtk_ctree_node_set_text( GTK_CTREE( tree ), GTK_CTREE_NODE( node ),
+				 1, text );
+	g_free( text );
+}
+
+void ascii_file( GtkWidget *widget )
+{
+	GladeXML *xml;
+	GtkWidget *menu;
+	gboolean active;
+	gchar *path;
+
+	GtkWidget *tree;
+	GtkWidget *node;
+	gchar *text;
+	gchar flags[ 3 ] = { '.', '.', '.' };
+
+	xml = glade_get_widget_tree( widget );
+	menu = glade_xml_get_widget( xml, "file_tree_menu" );
+
+	tree = gtk_object_get_data( GTK_OBJECT( menu ), "tree" );
+	node = gtk_object_get_data( GTK_OBJECT( menu ), "node" );
+	path = gtk_object_get_data( GTK_OBJECT( menu ), "path" );
+
+	active = screem_site_is_ascii( current_site, path );
+
+	if( ! active )
+		screem_site_add_ascii( current_site, path );
+	else
+		screem_site_remove_ascii( current_site, path );
+
+	if( screem_site_is_excluded( current_site, path ) )
+		flags[ 0 ] = 'E';
+	if( screem_site_is_ignored( current_site, path ) )
+		flags[ 1 ] = 'I';
+	if( screem_site_is_ascii( current_site, path ) )
+		flags[ 2 ] = 'A';
+	
+	text = g_strdup_printf( "%c%c%c", flags[ 0 ], flags[ 1 ], flags[ 2 ] );
+	gtk_ctree_node_set_text( GTK_CTREE( tree ), GTK_CTREE_NODE( node ),
+				 1, text );
+	g_free( text );
+}
+
+void ignore_file( GtkWidget *widget )
+{
+	GladeXML *xml;
+	GtkWidget *menu;
+	gboolean active;
+	gchar *path;
+
+
+	GtkWidget *tree;
+	GtkWidget *node;
+	gchar *text;
+	gchar flags[ 3 ] = { '.', '.', '.' };
+
+	xml = glade_get_widget_tree( widget );
+	menu = glade_xml_get_widget( xml, "file_tree_menu" );
+
+	tree = gtk_object_get_data( GTK_OBJECT( menu ), "tree" );
+	node = gtk_object_get_data( GTK_OBJECT( menu ), "node" );
+	path = gtk_object_get_data( GTK_OBJECT( menu ), "path" );
+
+	active = screem_site_is_ignored( current_site, path );
+
+	if( ! active )
+		screem_site_add_ignore( current_site, path );
+	else
+		screem_site_remove_ignore( current_site, path );
+
+	if( screem_site_is_excluded( current_site, path ) )
+		flags[ 0 ] = 'E';
+	if( screem_site_is_ignored( current_site, path ) )
+		flags[ 1 ] = 'I';
+	if( screem_site_is_ascii( current_site, path ) )
+		flags[ 2 ] = 'A';
+	
+	text = g_strdup_printf( "%c%c%c", flags[ 0 ], flags[ 1 ], flags[ 2 ] );
+	gtk_ctree_node_set_text( GTK_CTREE( tree ), GTK_CTREE_NODE( node ),
+				 1, text );
+	g_free( text );
+}
+
 static void tree_item_clicked( GtkCTree *tree, GtkCTreeNode *row, gint column )
 {
 	Page *page = NULL;
@@ -326,12 +521,6 @@ static void tree_item_clicked( GtkCTree *tree, GtkCTreeNode *row, gint column )
 	gchar **params;
 	gint num = 0;
 
-	const gchar *page_path;
-	gchar *pname;
-
-	GtkWidget *box = gtk_object_get_data( GTK_OBJECT( app ), "tab_box" );
-	GtkWidget *tab;
-
 	if( ! current_site )
 		return;
 
@@ -346,11 +535,16 @@ static void tree_item_clicked( GtkCTree *tree, GtkCTreeNode *row, gint column )
 	/* locate the page with the matching path */
 	page = screem_site_locate_page( current_site, path );
 
+#ifndef HAVE_GNOME_VFS
 	mime_type = gnome_mime_type( path );
+#else
+	mime_type = gnome_vfs_mime_type_from_name( path );
+#endif
 
 	/* if the path wasn't a page then we open it in the program
 	   determined by its mime type */
 	if( ! page ) {
+		/* FIXME: use gnome vfs is possible */
 		mime_exec = gnome_mime_program( mime_type );
 		if( ! mime_exec )
 			return;
@@ -378,7 +572,9 @@ static void tree_item_clicked( GtkCTree *tree, GtkCTreeNode *row, gint column )
 	if( page == prev )
 		return;
 
-
+	/* buffer any previous page that may be open in the editor */
+	if( screem_site_get_current_page( current_site ) )
+		screem_editor_buffer_text();
 
 	screem_page_insert( page );
 }
@@ -391,6 +587,7 @@ void tree_set_dnd_data( GtkWidget *widget, GdkDragContext *context,
 	GtkCTreeNode *node;
         gchar *path = NULL;
         gchar *text = NULL;
+	gchar *dir = NULL;
 	gint row;
 
         if( ! GTK_CLIST( widget )->selection )
@@ -403,11 +600,15 @@ void tree_set_dnd_data( GtkWidget *widget, GdkDragContext *context,
 							    node );
 	} else {
 		row = (gint)gtk_object_get_data(GTK_OBJECT(widget), "drag_start");
-		path = gtk_clist_get_row_data( GTK_CLIST( widget ), row );
+
+		dir = gtk_object_get_data( GTK_OBJECT( widget ), "current_dir" );
+		gtk_clist_get_text( GTK_CLIST( widget ), row, 1, &path );
+		path = g_strconcat( dir, G_DIR_SEPARATOR_S, path, NULL );
+		/* FIXME: Mem Leak */
 	}
 	
-        if( path ) {
-                switch( info ) {
+        if( path && ( *path != '\0' ) ) {
+		switch( info ) {
                 case TARGET_URI_LIST:
                         text = g_strconcat( "file:", path, "\r\n", NULL );
                         break;
@@ -439,8 +640,7 @@ void tree_drag_begin( GtkWidget *widget, GdkDragContext *context,
 		gtk_ctree_select( GTK_CTREE( widget ), node );
 	} else {
 		gtk_clist_select_row( GTK_CLIST( widget ), 
-				      GPOINTER_TO_INT( key ),
-				      0 );
+				      GPOINTER_TO_INT( key ), 0 );
 }
 }
 
@@ -463,7 +663,7 @@ gboolean tree_drag_motion( GtkWidget *widget,
 
         gdk_drag_status( context, action, time );
 
-        return TRUE;
+	return TRUE;
 }
 
 void tree_drop_data( GtkWidget *widget, GdkDragContext *context,
@@ -481,7 +681,6 @@ void tree_drop_data( GtkWidget *widget, GdkDragContext *context,
 	GtkCTreeNode *node = NULL;
 	gchar *path;
     	gchar *file;
-	gint len;
 	gboolean res;
 
 	gboolean is_dir;
@@ -563,16 +762,8 @@ void tree_drop_data( GtkWidget *widget, GdkDragContext *context,
 		if( ! strcmp( text, path ) )
 			return;
 
-		/* 
-		 * path == the directory to drop onto,
-		 * now get the file / directory name to drop
-		 * (not the full path for it though)
-		 */
-		file = text + strlen( text ) - ( 1 + is_dir );
-		while( *file != G_DIR_SEPARATOR )
-			file --;
-			
-		path = g_strconcat( path, ++ file, NULL );
+		file = g_basename( text );
+		path = g_strconcat( path, G_DIR_SEPARATOR_S, file, NULL );
 
 		/* check to see if text and path are the same now */
 		if( ! strcmp( text, path ) ) {
@@ -580,7 +771,7 @@ void tree_drop_data( GtkWidget *widget, GdkDragContext *context,
 			return;
 		}
 
-		if( is_dir )
+    		if( is_dir )
 			res = copy_dir( text, path, FALSE );
 		else
 			res = copy_file( text, path );
@@ -604,8 +795,8 @@ void tree_drop_data( GtkWidget *widget, GdkDragContext *context,
 
 			if( ( page = screem_site_get_current_page( site ) ) ) {
 				/* insert the possibly changed page */
-				screem_editor_clear();
-				screem_editor_insert( 0, page->data );
+				screem_editor_display_page( page );
+
 			}
 		}
 		g_free( path );
@@ -618,33 +809,27 @@ void tree_drop_data( GtkWidget *widget, GdkDragContext *context,
 
 }
 
-static void tree_open_file()
+void tree_open_file()
 {
 	GtkWidget *file_tree;
-	gpointer key;
         GtkCTreeNode *node;
 
 	file_tree = gtk_object_get_data( GTK_OBJECT( app ), "file_tree" );
 
-        key = gtk_object_get_data( GTK_OBJECT( file_tree ), "drag_start" );
+	node = gtk_object_get_data( GTK_OBJECT( file_tree ), "node" );
 
-        if( ! key )
-                return;
-
-        node = gtk_ctree_node_nth( GTK_CTREE( file_tree ), 
-				   GPOINTER_TO_INT( key ) );
         if( ! node )
                 return;
 
 	tree_item_clicked( GTK_CTREE( file_tree ), node, 0 );
 }
 
-static void tree_delete_file()
+void tree_delete_file()
 {
 	GtkWidget *file_tree;
 	GtkWidget *box;
-        const gchar *file = _( "Delete selected file" );
-        const gchar *dir =  _( "Delete selected directory" );
+        const gchar *file = _("Delete selected file");
+        const gchar *dir =  _("Delete selected directory");
         const gchar *message;
 
         gpointer key;
@@ -702,7 +887,7 @@ static void tree_delete_file()
 
 	/* perform cvs delete? */
 	if( screem_site_get_cvs_root( site ) ) {
-		box = gnome_message_box_new( _( "perform CVS delete?" ),
+		box = gnome_message_box_new( _("perform CVS delete?"),
 					     GNOME_MESSAGE_BOX_QUESTION,
 					     GNOME_STOCK_BUTTON_YES,
 					     GNOME_STOCK_BUTTON_NO, NULL );
@@ -720,7 +905,7 @@ static void tree_delete_file()
 	refresh_file_tree();
 }
 
-static void tree_rename_file()
+void tree_rename_file()
 {
 	GtkWidget *file_tree;
 	GtkWidget *dialog;
@@ -730,19 +915,18 @@ static void tree_rename_file()
 
 	GtkWidget *box;
 
-        const gchar *file = _( "Old file" );
-        const gchar *file2 = _( "New file" );
-        const gchar *dir =  _( "Old directory" );
-        const gchar *dir2 = _( "New directory" );
-        const gchar *fileTitle = _( "Rename file" );
-        const gchar *dirTitle = _( "Rename directory" );
+        const gchar *file = _("Old file");
+        const gchar *file2 = _("New file");
+        const gchar *dir =  _("Old directory");
+        const gchar *dir2 = _("New directory");
+        const gchar *fileTitle = _("Rename file");
+        const gchar *dirTitle = _("Rename directory");
         const gchar *title;
         const gchar *message;
         const gchar *message2;
 
 	const gchar *site_path;
  
-        gpointer key;
         GtkCTreeNode *node;
         gchar *path;
 	gint button;
@@ -750,14 +934,9 @@ static void tree_rename_file()
 	gchar *new_path;
 	gchar *temp;
 	gboolean status;
-	Page *page;
 	Page *cpage;
 
 	gboolean is_dir;
-	gint tab;
-	GtkWidget *pagebook;
-	GtkWidget *note;
-	gchar *pname;
 
 	site = current_site;
 
@@ -767,12 +946,8 @@ static void tree_rename_file()
 
 	file_tree = gtk_object_get_data( GTK_OBJECT( app ), "file_tree" );
 
-        key = gtk_object_get_data( GTK_OBJECT( file_tree ),
-                                   "drag_start" );
-        if( ! key )
-                return;
-        node = gtk_ctree_node_nth( GTK_CTREE( file_tree ), 
-				   GPOINTER_TO_INT( key ) );
+	node = gtk_object_get_data( GTK_OBJECT( file_tree ), "node" );
+  
         if( ! node )
                 return;
 
@@ -860,7 +1035,7 @@ static void tree_rename_file()
 		screem_editor_buffer_text();
        
 	screem_site_file_change( site, path, new_path );
-	
+
 	if( ! is_dir )
 		delete_file( path );
 	else
@@ -869,13 +1044,12 @@ static void tree_rename_file()
 	if( cpage ) {
 		/* we had a page open in the editor, show
 		   the possible changes to it */
-		screem_editor_clear();
-		screem_editor_insert( 0, cpage->data );
+		screem_editor_display_page( cpage );
 	}
 
 	/* perform cvs delete + cvs add ? */
 	if( screem_site_get_cvs_root( site ) ) {
-		box = gnome_message_box_new( _( "Rename in CVS?" ),
+		box = gnome_message_box_new( _("Rename in CVS?"),
 					     GNOME_MESSAGE_BOX_QUESTION,
 					     GNOME_STOCK_BUTTON_YES,
 					     GNOME_STOCK_BUTTON_NO, NULL );
@@ -904,12 +1078,8 @@ void screem_browse_list_show_dir( gchar *dir )
 	const gchar *mime_type;
 	gchar *item[ 2 ] = { NULL, NULL };
 	gint row;
-	gint ld;
-	gint data_row;
 	gboolean is_dir;
 
-	gchar *path;
-
 	gchar cwd[ 16384 ];
 	gchar cwd2[ 16384 ];
 
@@ -921,6 +1091,8 @@ void screem_browse_list_show_dir( gchar *dir )
 
 	getcwd( cwd, 16384 );
 
+	gtk_object_set_data( GTK_OBJECT( list ), "current_dir", 
+			     g_strdup( dir ) );
 	d = opendir( dir );
 	if( d ) {
 		chdir( dir );
@@ -929,44 +1101,56 @@ void screem_browse_list_show_dir( gchar *dir )
 		gtk_clist_freeze( GTK_CLIST( list ) );
 		gtk_clist_clear( GTK_CLIST( list ) );
 
-		for( ld = 0, row = 0; ( entry = readdir( d ) ); row ++ ) {
+		gtk_clist_set_sort_column( GTK_CLIST( list ), 1 );
+		gtk_clist_set_sort_type( GTK_CLIST( list ), GTK_SORT_ASCENDING );
+		gtk_clist_set_auto_sort( GTK_CLIST( list ), TRUE );
+
+		while( ( entry = readdir( d ) ) ) {
 			name = entry->d_name;
 			if( ! strcmp( ".", name ) ) {
 				name = dir;
+				continue;;
 			} else if( strcmp( "..", name ) && name[ 0 ] == '.' ) {
-				row --;
+				/* don't show hidden files */
 				continue;
 			}
 			item[ 1 ] = g_strdup( name );
 			is_dir = g_file_test( name, G_FILE_TEST_ISDIR );
 			if( ! is_dir ) {
+#ifndef HAVE_GNOME_VFS
 				mime_type = gnome_mime_type( name );
-				icon = icon_from_mime_type( mime_type );
-				gtk_clist_append( GTK_CLIST( list ), item );
+#else
+				mime_type = gnome_vfs_mime_type_from_name( name );
+#endif
+				icon = icon_from_mime_type( NULL, mime_type );
+				row = gtk_clist_append( GTK_CLIST( list ), 
+							item );
 				gtk_clist_set_pixmap( GTK_CLIST( list ),
 						      row, 0, icon->open,
 						      icon->open_mask );
-				data_row = row;
 			} else {
-				gtk_clist_insert( GTK_CLIST( list ), ld,item );
+				if( *item[1] != '/' ) {
+					g_free( item[ 1 ] );
+					item[ 1 ] = g_strconcat( "/", name, NULL );
+				}
+				row = gtk_clist_insert( GTK_CLIST( list ), 0,
+							item );
 				gtk_clist_set_pixmap( GTK_CLIST( list ),
-						      ld, 0, folder_close,
+						      row, 0, folder_close,
 						      folder_close_mask );
-				data_row = ld;
-				ld ++;
 			}
-			if( strcmp( name, dir ) )
-				path = g_strconcat(dir, G_DIR_SEPARATOR_S,
-						   name, NULL);
-			else
-				path = g_strdup( dir );
-			gtk_clist_set_row_data( GTK_CLIST( list ), data_row,
-						path );
+			g_free( item[ 1 ] );
 		}
 		gtk_clist_columns_autosize( GTK_CLIST( list ) );
+		/* switch auto sort off so we can show the cwd name
+		   as the top node */
+		gtk_clist_set_auto_sort( GTK_CLIST( list ), FALSE );
+		item[ 1 ] = dir;
+		row = gtk_clist_prepend( GTK_CLIST( list ), item );
+		gtk_clist_set_pixmap( GTK_CLIST( list ), row, 0,
+				      folder_close, folder_close_mask );
 		gtk_clist_thaw( GTK_CLIST( list ) );
 		closedir( d );
-		gtk_object_set_data( GTK_OBJECT( list ), "current_dir", dir );
 	}
 
 	chdir( cwd );
@@ -975,14 +1159,11 @@ void screem_browse_list_show_dir( gchar *dir )
 void browse_list_clicked( GtkWidget *widget, GdkEventButton *event,
 			  gpointer data )
 {
-	GtkCTreeNode *node = NULL;
-        GtkWidget *menu;
         gint row = -1;
         gint col;
 
 	gchar *path;
-
-	static guint32 time = 0;
+	gchar *dir;
 
 	gtk_clist_get_selection_info( GTK_CLIST( widget ),
                                       event->x, event->y, &row, &col );
@@ -997,29 +1178,206 @@ void browse_list_clicked( GtkWidget *widget, GdkEventButton *event,
 
 	/* what button was clicked? */
 	if( event->button == 1 && row ) {
-		if( time == event->time ) {
-			path = gtk_clist_get_row_data( GTK_CLIST( widget ),
-						       row );
+		if( event->type == GDK_2BUTTON_PRESS ) {
+			dir = gtk_object_get_data( GTK_OBJECT( widget ), "current_dir" );
+			gtk_clist_get_text( GTK_CLIST( widget ), row, 1, &path );
+			path = g_strconcat( dir, G_DIR_SEPARATOR_S, path, NULL );
+
 			if( g_file_test( path, G_FILE_TEST_ISDIR ) )
 				screem_browse_list_show_dir( path );
-			else {
-
+			else if( screem_page_is_file_page( path ) ){
+				/* switch to fake site and open */
+				screem_site_switch_to_site(_("Single Pages"));
+				screem_page_open_with_filename( path );
 			}
+			g_free( path );
 		}
-                time = event->time;
-        } 
+	} 
 }
 
 
-void resource_list_display()
+void resource_list_display( GtkWidget *tree )
 {
 	const gchar *resource_root = RESOURCE_PATH;
 	xmlDocPtr doc;
-	GtkWidget *tree;
+      	gint number;
+	gint pos;
+        struct dirent **namelist;
+	gchar *path;
+	gchar *name;
 
-	doc = build_directory_tree( NULL, resource_root, NULL );
+	number = scandir( resource_root, &namelist, 0, alphasort );
+
+	if( number < 0 )
+		return;
 
-	tree = gtk_object_get_data( GTK_OBJECT( app ), "resource_tree" );
 	gtk_clist_clear( GTK_CLIST( tree ) );
-	build_file_tree( tree, NULL, doc );
+
+	for( pos = 0; pos < number; pos ++ ) {
+		name = namelist[ pos ]->d_name;
+		if( strlen( name ) > 2 ) {
+			path = g_strconcat( resource_root, name, NULL );
+			doc = build_directory_tree( NULL, path, NULL );
+			build_file_tree( tree, NULL, doc );
+			xmlFreeDoc( doc );
+			g_free( path );
+		}
+		free( namelist[ pos ] );
+	}
+
+	/* now add the tag tree in as a resource */
+	gtk_clist_freeze( GTK_CLIST( tree ) );
+	screem_tag_tree_build( tree, cfg->tag_tree );
+	gtk_clist_thaw( GTK_CLIST( tree ) );
+}
+
+void tree_file_info()
+{
+	GtkWidget *file_tree;
+        GtkCTreeNode *node;
+	gchar *path;
+
+	file_tree = gtk_object_get_data( GTK_OBJECT( app ), "file_tree" );
+
+	node = gtk_object_get_data( GTK_OBJECT( file_tree ), "node" );
+
+        if( ! node )
+                return;
+
+        path = (gchar*)gtk_ctree_node_get_row_data( GTK_CTREE( file_tree ),
+                                                    node );
+
+	file_information( path );
+}
+
+void file_information( const gchar *filename )
+{
+	struct stat s;
+	GladeXML *xml;
+	GtkWidget *widget;
+	GtkWidget *i;
+	gchar *temp;
+	Site *site;
+	Icons *icon;
+	gchar *t;
+	const gchar *mime_type;
+	GdkPixmap *open = NULL;
+	GdkBitmap *open_mask = NULL;
+
+	uid_t user_id = geteuid();
+	struct passwd *userinfo;
+	struct group *groupinfo;
+	
+	site = current_site;
+
+	if( stat( filename, &s ) < 0 )
+		return;
+
+	xml = glade_xml_new( cfg->glade_path, "file_info" );
+
+	widget = glade_xml_get_widget( xml, "local" );
+	gtk_label_set( GTK_LABEL( widget ), filename );
+
+	widget = glade_xml_get_widget( xml, "mime" );
+	i = glade_xml_get_widget( xml, "file_icon" );
+
+	if( S_ISDIR(s.st_mode) ) {
+		mime_type = "Directory";
+		gtk_pixmap_set( GTK_PIXMAP( i ), folder_close,
+				folder_close_mask );
+	} else {
+#ifndef HAVE_GNOME_VFS
+		mime_type = gnome_mime_type( filename );
+#else
+		mime_type = gnome_vfs_mime_type_from_name( filename );
+#endif
+		icon = icon_from_mime_type( filename, mime_type );
+		if( icon->pixbuf ) {
+			gdk_pixbuf_render_pixmap_and_mask( icon->pixbuf,
+							   &open, &open_mask,
+							   100 );
+			gtk_pixmap_set( GTK_PIXMAP( i ), open, open_mask );
+		}
+	}
+	gtk_label_set( GTK_LABEL( widget ), mime_type );
+
+	widget = glade_xml_get_widget( xml, "mod" );
+	t = g_strdup( ctime( &s.st_mtime ) );
+	t = g_strchomp( t );
+	gtk_label_set( GTK_LABEL( widget ), t );
+
+	widget = glade_xml_get_widget( xml, "change" );
+	t = g_strdup( ctime( &s.st_ctime ) );
+	t = g_strchomp( t );
+	gtk_label_set( GTK_LABEL( widget ), t );
+
+	widget = glade_xml_get_widget( xml, "access" );
+	t = g_strdup( ctime( &s.st_atime ) );
+	t = g_strchomp( t );
+	gtk_label_set( GTK_LABEL( widget ), t );
+
+	widget = glade_xml_get_widget( xml, "uid" );
+	temp = g_strdup_printf( "%i", s.st_uid );
+	userinfo = getpwuid( s.st_uid );
+	if( userinfo ) {
+		gtk_entry_set_text( GTK_ENTRY( widget ), userinfo->pw_name );
+	} else {
+		gtk_entry_set_text( GTK_ENTRY( widget ), temp );
+	}
+	g_free( temp );
+
+	widget = glade_xml_get_widget( xml, "gid" );
+	temp = g_strdup_printf( "%i", s.st_gid );
+	groupinfo = getgrgid( s.st_gid );
+	if( groupinfo ) {
+		gtk_entry_set_text( GTK_ENTRY( widget ), groupinfo->gr_name );
+	} else {
+		gtk_entry_set_text( GTK_ENTRY( widget ), temp );
+	}
+	g_free( temp );
+
+	widget = glade_xml_get_widget( xml, "size" );
+	temp = g_strdup_printf( "%i KBytes (%i bytes)", 
+				((int)s.st_size) / 1024,
+				(int)s.st_size );
+	gtk_label_set( GTK_LABEL( widget ), temp );
+	g_free( temp );
+
+	widget = glade_xml_get_widget( xml, "file_exclude" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+				      screem_site_is_excluded(site,filename) );
+	widget = glade_xml_get_widget( xml, "file_ignore" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+				      screem_site_is_ignored(site, filename) );
+	widget = glade_xml_get_widget( xml, "file_ascii" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+				      screem_site_is_ascii( site, filename ) );
+
+	widget = glade_xml_get_widget( xml, "file_info" );
+	gtk_widget_show( widget );
+
+	gnome_dialog_run_and_close( GNOME_DIALOG( widget ) );
+}
+
+/* Process a directory name, and if one was given, create it. */
+void create_dir_callback( gchar *string, gpointer data )
+{
+	if( ! string ) {
+		screem_show_warning( _("No directory name specified.") );
+		return;
+	} else if( ! mkdir_recursive( string ) ) {
+		screem_show_warning( _( "Directory could not be created." ) );
+		return;
+	}
+	/* Directory was created just fine, refresh the file view */
+	refresh_file_tree();
+}
+ 
+/* Request a directory name from the user in a gnome-dialog */
+void screem_site_create_dir()
+{
+	gnome_app_request_string( GNOME_APP( app ),
+				  _( "Enter the directory name to create:" ),
+				  (GnomeStringCallback)create_dir_callback,
+				  NULL );
 }
